// @madex/ex-ts-dao 是 ts 的 dao, 代码在 bitbucket/ex-js-dao 的 ts 分支上
import { coinType, ormDB, spotPairs, mixinCoin, mixinChain,nftCollection } from "@madex/ex-ts-dao";
import { addCoin2Core } from "../../../utils/coreSystemUtils";
import { ErrorCode } from "../../../constant/errorCode";
import { addOptLog } from "./userOptLog.service";

const _ = require("lodash");

let { logger, datetimeUtils } = require('@madex/ex-js-public');

export interface ListParam {
    symbol: string | any;

    page: number;

    size: number;

    status?: number

    from_time?: Date | any

    to_time?: Date | any
}

export interface AddParam {
    id: any,

    symbol: string;

    mixin_id: string;

    asset_id: number;

    is_active: number;

    is_main: number;

    name: string;

    general_name: string;

    chain_type?: string;

    issupport_memo?: number;

    original_decimals: number;

    valid_decimals: number;

    contract?: string;

    contract_father?: string;

    enable_deposit?: number;

    enable_withdraw?: number;

    enable_transfer?: number;

    withdraw_fee?: number;

    withdraw_min?: number;

    deposit_min?: number;

    deposit_confirm_count: number;

    safe_confirm_count: number;

    explor_url?: string;

    icon_url?: string;

    describe_url?: string;

    weight?: number;

    forbid_info?: string;

    describe_summary?: string;

    total_amount?: number;


    supply_amount?: number;

    circulation_rate?: number;

    address_num?: number;

    price?: string;

    supply_time?: string;

    deflation_rate?: number;

    comment?: string;

    max_transfer_amount?: number;

    tag: string;

    forbid_info_related?: number;

    is_hidden?: number;

    main_status?: number;

    createdAt?: Date | any,

    updatedAt?: Date | any,
}


export async function list(param: ListParam) {
    let where = {};
    if (param.symbol) {
        where["symbol"] = { [ormDB.Op.like]: `%${param.symbol}%` };
    }
    if (param.status || param.status === 0) {
        where["main_status"] = param.status
    }
    if (param.from_time && param.to_time) {
        where['createdAt'] = { [ormDB.Op.between]: [param.from_time, param.to_time] }
    }
    let resList = await coinType.prototype.findAndCount({
        where: where,
        limit: param.size,
        offset: (param.page - 1) * param.size,
        order: [["id", "desc"]],
        raw: true
    });
    return resList;
}

export async function chainList() {
    let resList = await coinType.prototype.findAll({
        attributes: ['chain_type'],
        where: {
            is_main: 1
        },
        raw: true
    });
    let chains: any[] = [];
    for (let item of resList) {
        let chainType = item.chain_type;
        if (chainType) {
            chains.push(chainType);
        }
    }
    return chains;
}

export const save = async (param: AddParam, currentUserId: any, ip: any) => {
    if (!param.weight) {
        param.weight = 0;
    }
    param.updatedAt = new Date();
    param.createdAt = new Date();
    let general_name = param.general_name;
    let exist = await coinType.prototype.findOne({ // 同名币判断
        where: {
            is_main: 1,
            general_name: general_name,
        },
        raw: true
    });

    if (exist) {
        param.is_main = 0;
        param.asset_id = exist.asset_id;
        param.main_status = exist.main_status;
    }
    else {
        param.is_main = 1;
        param.asset_id = 0;
        param.main_status = 0;
    }
    await coinType.prototype.create(param);
    //管理后台操作日志
    addOptLog(currentUserId, 0, '币种管理新增', ip, JSON.stringify(param), '币种管理');
    return 'ok';
};

export const update = async (param: AddParam, currentUserId: any, ip: any) => {
    let id = param.id;
    let dbInfo = await coinType.prototype.findOne({
        where: {
            id: id
        },
        raw: true
    });
    if (!dbInfo) {
        throw ErrorCode.DATA_NOT_EXIST
    }
    if (dbInfo.audit_status) {//审核通过后不可修改
        throw ErrorCode.STATUS_PASS_NO_UP
    }
    param.updatedAt = new Date();
    await coinType.prototype.update(param, {
        where: { id: id }
    })
    //管理后台操作日志
    addOptLog(currentUserId, 0, '币种管理更新', ip, JSON.stringify(param), '币种管理');
    return 'ok';
};

export const pushToCoreSystem = async (id: any, currentUserId: any, ip: any) => {

    let where = { id: id, main_status: 0, is_main: 1 };

    let cfg = await coinType.prototype.find({
        where: where, raw: true
    });
    if (!cfg) {
        throw ErrorCode.SUBMIT_STATUS_OR_MAIN_ERR;
    }
    if (!cfg.audit_status) {
        throw ErrorCode.AFTER_AUDIT_SUBMIT;
    }
    let symbol = cfg.general_name;

    let optResult = await addCoin2Core(symbol, cfg.id);
    if (!optResult.is_success) {
        //管理后台操作日志
        addOptLog(currentUserId, 0, '提交币种到撮合-失败', ip, `symbol:${symbol},error_msg:${optResult.err_msg}`, '币种管理');
        throw ErrorCode.ADD_PAIR_TO_CORE_ERR;
    }
    await coinType.prototype.update({ main_status: 1 }, {
        where: where
    });
    //管理后台操作日志
    addOptLog(currentUserId, 0, '提交币种到撮合-成功', ip, `symbol:${symbol}`, '币种管理');
    return 'ok';

};

export async function getAllSubmitSuccess() {
    let resList = await coinType.prototype.findAll({
        where: {
            main_status: 2
        },
        order: [["id", "desc"]],
        raw: true
    });
    return resList;
}


export const audit = async (param: AddParam, currentUserId: any, ip: any) => {
    let id = param.id;
    let dbInfo = await coinType.prototype.findOne({
        where: {
            id: id
        },
        raw: true
    });
    if (!dbInfo) {
        throw ErrorCode.DATA_NOT_EXIST
    }
    param.updatedAt = new Date();
    if (!dbInfo.audit_status) {
        await coinType.prototype.update({
            audit_status: 1,
        }, {
            where: { id: id }
        })
        //管理后台操作日志
        addOptLog(currentUserId, 0, '币种管理审核', ip, JSON.stringify(param), '币种管理');
    }

    return 'ok';
};


export async function mixinCoinList(param: ListParam) {
    let where = {};
    if (param.symbol) {
        where["symbol"] = { [ormDB.Op.like]: `%${param.symbol}%` };
    }
    let resList = await mixinCoin.prototype.findAndCount({
        attributes: ['symbol', 'name', 'symbol_id', ['symbol_id', 'mixin_id'], 'display_symbol', 'chain_id',
            ['address', 'contract'], ['precision', 'original_decimals'], ['dust', 'deposit_min'], ['dust', 'withdraw_min'],
            ['confirm', 'deposit_confirm_count'], ['confirm', 'safe_confirm_count'],
            'price_btc', 'price_usd', 'balance', 'comment'],
        // include:{
        //     model:mixinChain.prototype,
        //     required:true,   //INNER JOIN
        //     attributes:['name','symbol','threshold','withdrawal_fee','is_memo'],
        // },
        where: where,
        limit: param.size,
        offset: (param.page - 1) * param.size,
        order: [["symbol", "asc"]],
        raw: true
    });
    let chainIDs = _.map(resList.rows, 'chain_id')

    let m = await mixinChain.prototype.findAll({
        attributes: ['name', 'symbol', 'threshold', 'withdrawal_fee', 'is_memo', 'chain_id'],
        where: {
            chain_id: chainIDs
        },
        raw: true,
    })
    let chainMap = _.keyBy(m, 'chain_id');

    for (const item of resList.rows) {
        let isMainToken = (item.symbol_id == item.chain_id && item.chain_id == item.contract)
        if (chainMap[item.chain_id]) {
            item.chain_type = chainMap[item.chain_id].symbol
            item.issupport_memo = chainMap[item.chain_id].is_memo;
            item.valid_decimals = 8;
            item.withdraw_fee = isMainToken ? chainMap[item.chain_id].withdrawal_fee : (5 / (item.price_usd > 0 ? item.price_usd : 0.001)).toFixed(4);
        }
        if (item.contract) {
            item.is_erc20 = 1;
        }
        else {
            item.is_erc20 = 0;
        }
    }
    return resList;
}

export async function nftList() {
    let resList = await nftCollection.prototype.findAndCount({
        attributes:['id','name','icon_url','symbol','chain_type'],
        order: [["id", "desc"]],
        raw: true
    });
    return resList;

}

export const updateIconUrl = async (param: AddParam, currentUserId: any, ip: any) => {
    let id = param.id;
    let dbInfo = await coinType.prototype.findOne({
        where: {
            id: id
        },
        raw: true
    });
    if (!dbInfo) {
        throw ErrorCode.DATA_NOT_EXIST
    }
    let upInfo = {
        icon_url: param.icon_url,
        updatedAt: new Date(),
    }
    await coinType.prototype.update(upInfo, {
        where: { id: id }
    })
    //管理后台操作日志
    addOptLog(currentUserId, 0, '币种管理更新 icon_url', ip, JSON.stringify(param), '币种管理');
    return 'ok';
};