/**
 * Madex 管理后台登录过滤
 */
import Config from "../../config";
const {
    Res3Utils,
    logger: Logger,
} = require("@madex/ex-js-public");
import * as ReqUtils from "../utils/req-utils"

const CLASS_NAME = "login-filter";

const ExcludeApi = {
    "user/login": 1,
    "user/logout": 1,
    "user/login/confirm": 1,
    "mUser/fee/vip/level/list": 1,
    "spotPair/getAllSubmitSuccess": 1,
    "contractPair/getAllSubmitSuccess": 1,
    "coinType/getAllSubmitSuccess": 1,
    'acl/role/getAll': 1,
    'position/allList': 1,
    'department/allList': 1,
    'operate/other/business/area/list': 1,
    'coinType/mixinCoinList': 1,
    'coinType/nftList': 1,
    'coinType/chainList': 1,
    'mUser/manage/billTypeList': 1,

};
//管理员需要强制绑定，该位置做特殊处理
const AdminExcludeApi = {
    "user/bind/totp/ask": 1,
    "user/bind/totp/confirm": 1,
}

let filter = function (app: any) {
    app.use(function (req, res, next) {
        let path = ReqUtils.parsePath(req.originalUrl);
        try {
            // Madex 资产管理后台的接口 跳过
            if (ExcludeApi[path] || path.startsWith(Config.BASE_ABK_URL)) {
                next();
            }
            else {
                let cookies = req.cookies
                ReqUtils
                    .checkCookie(cookies, AdminExcludeApi[path], path, req)
                    .then(() => next())
                    .catch(e => res.json(Res3Utils.getErrorResult(e)))
            }
        }
        catch (e) {
            Logger.error(`${CLASS_NAME} filter error!! - ${e}`);
            return res.json(Res3Utils.getErrorResult(e));
        }
    });
};

module.exports = {
    filter: filter,
};
